package com.qx.gpsp.internal;

import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.apache.http.client.methods.CloseableHttpResponse;
import org.json.JSONObject;

import com.qx.gpsp.common.WzBizType;
import com.qx.gpsp.common.WzConstants;
import com.qx.gpsp.common.WzException;
import com.qx.gpsp.manager.WzEntityManager;
import com.qx.gpsp.model.WzEntity;
import com.qx.gpsp.model.result.WzCreateEntityResult;
import com.qx.gpsp.model.result.WzDeleteEntityResult;
import com.qx.gpsp.model.result.WzQueryEntitiesByAppResult;
import com.qx.gpsp.model.result.WzQueryEntitiesResult;
import com.qx.gpsp.model.result.WzQueryEntityResult;
import com.qx.gpsp.model.result.WzUpdateEntityResult;



public class InternalWzEntityManager extends InternalWzManager implements WzEntityManager{

	public InternalWzEntityManager(String appKey, String appSecret) {
		super(appKey, appSecret);
	}
	
	/**
	 * 创建实体，此函数仅支持创建WzBiType.GENERAL业务类型的实体
	 * */
	
	@Override
	public WzCreateEntityResult createEntity(WzEntity wzEntity) throws WzException {
		if(wzEntity == null)
			throw new WzException(WzException.IllegalArgumentException,"wzEntity can't be null");
		
		JSONObject entityJson = new JSONObject();
		if(wzEntity.getEntityName() == null || wzEntity.getEntityName().isEmpty())
			throw new WzException(WzException.IllegalArgumentException,"wzEntity.entityName should not be null or empty");
		entityJson.put("entityName", wzEntity.getEntityName());
		
		if(wzEntity.getEntityType() == null || wzEntity.getEntityType().isEmpty())
			throw new WzException(WzException.IllegalArgumentException,"wzEntity.entityType should not be null or empty");
		entityJson.put("entityType", wzEntity.getEntityType());
		
		if(wzEntity.getAttributes() != null)
			entityJson.put("attributes", wzEntity.getAttributes());
		String entity = entityJson.toString();
		
		Map<String, String> paramMap = new HashMap<String, String>();
		paramMap.put("entity", entity);
	
		WzRequestManager wzRequestManager = new WzRequestManager(this.getAppKey(),this.getAppSecret());
		CloseableHttpResponse response = wzRequestManager.execute(WzConstants.CREATE_ENTITY,paramMap);
		return (WzCreateEntityResult) WzResponseManager.parseResponse(response, WzConstants.CREATE_ENTITY);
	}
	
	/**
	 * 删除实体
	 * */
	@Override
	public WzDeleteEntityResult deleteEntity(String bizType,String entityName) throws WzException {
		
		Map<String, String> paramMap = new HashMap<String, String>();
		
		if(bizType == null || bizType.isEmpty())
			paramMap.put("bizType", WzBizType.GENERAL);
		else if(WzBizType.hasBizType(bizType))
			paramMap.put("bizType", bizType);
		else
			throw new WzException(WzException.IllegalArgumentException,"bizType is illegal");
		
		if(entityName == null || entityName.isEmpty())
			throw new WzException(WzException.IllegalArgumentException,"wzEntity.entityType should not be null or empty");
		paramMap.put("entityName", entityName);
		
		WzRequestManager wzRequestManager = new WzRequestManager(this.getAppKey(),this.getAppSecret());
		CloseableHttpResponse response = wzRequestManager.execute(WzConstants.DELETE_ENTITY,paramMap);
		return (WzDeleteEntityResult) WzResponseManager.parseResponse(response, WzConstants.DELETE_ENTITY);

	}
	
	/**
	 * 更新实体，仅支持更新属性信息attributes
	 * */
	@Override
	public WzUpdateEntityResult updateEntity(WzEntity wzEntity) throws WzException {
		if(wzEntity == null)
			throw new IllegalArgumentException("wzEntity should not be null");
		
		Map<String, String> paramMap = new HashMap<String, String>();
		if(wzEntity.getBizType() != null && WzBizType.hasBizType(wzEntity.getBizType()))
			paramMap.put("bizType", wzEntity.getBizType());
		else
			throw new WzException(WzException.IllegalArgumentException,"wzEntity.bizType should be WzBizType.GENERAL or WzBizType.JT808");
		
		if(wzEntity.getEntityName() == null || wzEntity.getEntityName().isEmpty())
			throw new WzException(WzException.IllegalArgumentException,"wzEntity.entityName should not be null or empty");
		paramMap.put("entityName", wzEntity.getEntityName());
		
		if(wzEntity.getAttributes() != null && !wzEntity.getAttributes().isEmpty()){
			JSONObject jsonObject = new JSONObject();
			for (String key : wzEntity.getAttributes().keySet()) {  
				jsonObject.put(key,  wzEntity.getAttributes().get(key));
			}
			paramMap.put("attributes", jsonObject.toString());
		}	
		else
			throw new WzException(WzException.IllegalArgumentException,"wzEntity.entityName should not be null or empty");		
		
		WzRequestManager wzRequestManager = new WzRequestManager(this.getAppKey(),this.getAppSecret());
		CloseableHttpResponse response = wzRequestManager.execute(WzConstants.UPDATE_ENTITY,paramMap);
		return (WzUpdateEntityResult) WzResponseManager.parseResponse(response, WzConstants.UPDATE_ENTITY);

	}

	/**
	 * 查询实体
	 * */
	@Override
	public WzQueryEntityResult queryEntity(String bizType,String entityName) throws WzException {
		Map<String, String> paramMap = new HashMap<String, String>();
		if(WzBizType.hasBizType(bizType))
			paramMap.put("bizType",bizType);
		else if(bizType == null || bizType.isEmpty())
			paramMap.put("bizType",WzBizType.GENERAL);
		else
			throw new WzException(WzException.IllegalArgumentException,"bizType should be WzBizType.GENERAL or WzBizType.JT808");
		
		if(entityName == null || entityName.isEmpty())
			throw new WzException(WzException.IllegalArgumentException,"wzEntity.entityName should not be null or empty");
		paramMap.put("entityName",entityName);
		
		WzRequestManager wzRequestManager = new WzRequestManager(this.getAppKey(),this.getAppSecret());
		CloseableHttpResponse response = wzRequestManager.execute(WzConstants.QUERY_ENTITY,paramMap);
		return (WzQueryEntityResult) WzResponseManager.parseResponse(response, WzConstants.QUERY_ENTITY);
	}
	
	/**
	 * 查询多个实体
	 * */
	@Override
	public WzQueryEntitiesResult queryEntities(String bizType,List<String> entityNames) throws WzException {
		Map<String, String> paramMap = new HashMap<String, String>();
		if(WzBizType.hasBizType(bizType))
			paramMap.put("bizType",bizType);
		else if(bizType == null || bizType.isEmpty())
			paramMap.put("bizType",WzBizType.GENERAL);
		else
			throw new WzException(WzException.IllegalArgumentException,"bizType should be WzBizType.GENERAL or WzBizType.JT808");
		
		if(entityNames == null || entityNames.isEmpty())
			throw new WzException(WzException.IllegalArgumentException,"entityNames should not be null or empty");
		else
			paramMap.put("entityNames",JSONObject.valueToString(entityNames));
		
		WzRequestManager wzRequestManager = new WzRequestManager(this.getAppKey(),this.getAppSecret());
		CloseableHttpResponse response = wzRequestManager.execute(WzConstants.QUERY_ENTITIES,paramMap);
		return (WzQueryEntitiesResult) WzResponseManager.parseResponse(response, WzConstants.QUERY_ENTITIES);

	}

	@Override
	public WzQueryEntitiesByAppResult queryEntitiesByApp(String bizType,int pageNum,int pageSize) throws WzException {
		Map<String, String> paramMap = new HashMap<String, String>();
		if(WzBizType.hasBizType(bizType))
			paramMap.put("bizType",bizType);
		else if(bizType == null || bizType.isEmpty())
			paramMap.put("bizType",WzBizType.GENERAL);
		else
			throw new WzException(WzException.IllegalArgumentException,"bizType should be WzBizType.GENERAL or WzBizType.JT808");
		paramMap.put("pageNum", String.valueOf(pageNum));
		paramMap.put("pageSize", String.valueOf(pageSize));
		
		WzRequestManager wzRequestManager = new WzRequestManager(this.getAppKey(),this.getAppSecret());
		CloseableHttpResponse response = wzRequestManager.execute(WzConstants.QUERY_ENTITIES_BY_APP,paramMap);
		return (WzQueryEntitiesByAppResult) WzResponseManager.parseResponse(response, WzConstants.QUERY_ENTITIES_BY_APP);
	}
}
