package com.qx.gpsp.internal;

import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.apache.http.client.methods.CloseableHttpResponse;
import org.json.JSONObject;

import com.qx.gpsp.common.WzBizType;
import com.qx.gpsp.common.WzConstants;
import com.qx.gpsp.common.WzCoordinateType;
import com.qx.gpsp.common.WzException;
import com.qx.gpsp.manager.WzEntityPointManager;
import com.qx.gpsp.model.WzEntityPoint;
import com.qx.gpsp.model.result.WzAddPointResult;
import com.qx.gpsp.model.result.WzAddPointsResult;
import com.qx.gpsp.model.result.WzLocateResult;
import com.qx.gpsp.model.result.WzQueryPointsResult;

public class InternalWzEntityPointManager extends InternalWzManager implements WzEntityPointManager{

	public InternalWzEntityPointManager(String appKey, String appSecret) {
		super(appKey, appSecret);
		// TODO Auto-generated constructor stub
	}

	@Override
	public WzAddPointResult addPoint(WzEntityPoint wzEntityPoint) throws WzException {
		// TODO Auto-generated method stub
		Map<String, String> paramMap = new HashMap<String, String>();
		if(wzEntityPoint == null)
			throw new WzException(WzException.IllegalArgumentException,"wzEntityPoint should not be null");
		
		if(wzEntityPoint.getEntityName() == null || wzEntityPoint.getEntityName().isEmpty())
			throw new WzException(WzException.IllegalArgumentException,"wzEntityPoint.entityName should not be null or empty");
		
		if(wzEntityPoint.getLon()<-180 || wzEntityPoint.getLon() > 180)
			throw new WzException(WzException.IllegalArgumentException,"wzEntityPoint.lon should in range [-180,180]");
		
		if(wzEntityPoint.getLat() < -90 || wzEntityPoint.getLat() > 90)
			throw new WzException(WzException.IllegalArgumentException,"wzEntityPoint.lat should in range [-90,90]");
		
		if(wzEntityPoint.getCoordinateType() != null && !WzCoordinateType.hasCoordinateType(wzEntityPoint.getCoordinateType())){
			throw new WzException(WzException.IllegalArgumentException,"wzEntityPoint.coordinate should be WzCoorinateType.WGS84 or WzCoordinateType.GCJ02");
		}
		paramMap.put("point", wzEntityPoint.toString());
		
		WzRequestManager wzRequestManager = new WzRequestManager(this.getAppKey(),this.getAppSecret());
		CloseableHttpResponse response = wzRequestManager.execute(WzConstants.ADD_POINT,paramMap);
		return (WzAddPointResult) WzResponseManager.parseResponse(response, WzConstants.ADD_POINT);
	}

	@Override
	public WzAddPointsResult addPoints(List<WzEntityPoint> wzEntityPoints) throws WzException {
		Map<String, String> paramMap = new HashMap<String, String>();
		if(wzEntityPoints == null || wzEntityPoints.isEmpty())
			throw new WzException(WzException.IllegalArgumentException,"wzEntityPoints should not be null");
		paramMap.put("points", wzEntityPoints.toString());
		
		WzRequestManager wzRequestManager = new WzRequestManager(this.getAppKey(),this.getAppSecret());
		CloseableHttpResponse response = wzRequestManager.execute(WzConstants.ADD_POINTS,paramMap);
		return (WzAddPointsResult) WzResponseManager.parseResponse(response, WzConstants.ADD_POINTS);
	}

	@Override
	public WzLocateResult locate(String bizType, List<String> entityNames, int activeTime) throws WzException {
		// TODO Auto-generated method stub
		Map<String, String> paramMap = new HashMap<String, String>();
		if(WzBizType.hasBizType(bizType))
			paramMap.put("bizType",bizType);
		else if(bizType == null || bizType.isEmpty())
			paramMap.put("bizType",WzBizType.GENERAL);
		else
			throw new WzException(WzException.IllegalArgumentException,"bizType should be WzBizType.GENERAL or WzBizType.JT808");
		
		if(entityNames == null || entityNames.isEmpty())
			throw new WzException(WzException.IllegalArgumentException,"entityNames should not be null");
		else
			paramMap.put("entityNames",JSONObject.valueToString(entityNames));
		
		paramMap.put("activeTime", String.valueOf(activeTime));
		
		WzRequestManager wzRequestManager = new WzRequestManager(this.getAppKey(),this.getAppSecret());
		CloseableHttpResponse response = wzRequestManager.execute(WzConstants.LOCATE,paramMap);
		return (WzLocateResult) WzResponseManager.parseResponse(response, WzConstants.LOCATE);
	}

	@Override
	public WzQueryPointsResult queryPoints(String bizType, String entityName, long beginTime, long endTime, int pageNum,
			int pageSize) throws WzException {
		// TODO Auto-generated method stub
		Map<String, String> paramMap = new HashMap<String, String>();
		if(WzBizType.hasBizType(bizType))
			paramMap.put("bizType",bizType);
		else if(bizType == null || bizType.isEmpty())
			paramMap.put("bizType",WzBizType.GENERAL);
		else
			throw new WzException(WzException.IllegalArgumentException,"bizType should be WzBizType.GENERAL or WzBizType.JT808");
				
		if(entityName == null || entityName.isEmpty())
			throw new WzException(WzException.IllegalArgumentException,"entityName should not be null");
		
		paramMap.put("entityName", entityName);
		
		paramMap.put("beginTime", String.valueOf(beginTime));
		paramMap.put("endTime", String.valueOf(endTime));
		paramMap.put("pageNum", String.valueOf(pageNum));
		paramMap.put("pageSize", String.valueOf(pageSize));
				
		WzRequestManager wzRequestManager = new WzRequestManager(this.getAppKey(),this.getAppSecret());
		CloseableHttpResponse response = wzRequestManager.execute(WzConstants.QUERY_POINTS,paramMap);
		return (WzQueryPointsResult) WzResponseManager.parseResponse(response, WzConstants.QUERY_POINTS);

	}
}
