package com.qx.gpsp.internal;

import java.io.IOException;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

import org.apache.http.HttpEntity;
import org.apache.http.ParseException;
import org.apache.http.client.methods.CloseableHttpResponse;
import org.apache.http.util.EntityUtils;
import org.json.JSONArray;
import org.json.JSONObject;

import com.qx.gpsp.common.WzConstants;
import com.qx.gpsp.common.WzException;
import com.qx.gpsp.model.WzEntity;
import com.qx.gpsp.model.WzEntityPoint;
import com.qx.gpsp.model.WzResult;
import com.qx.gpsp.model.result.WzAddPointResult;
import com.qx.gpsp.model.result.WzAddPointsResult;
import com.qx.gpsp.model.result.WzCreateEntityResult;
import com.qx.gpsp.model.result.WzDeleteEntityResult;
import com.qx.gpsp.model.result.WzLocateResult;
import com.qx.gpsp.model.result.WzQueryEntitiesByAppResult;
import com.qx.gpsp.model.result.WzQueryEntitiesResult;
import com.qx.gpsp.model.result.WzQueryEntityResult;
import com.qx.gpsp.model.result.WzQueryPointsResult;
import com.qx.gpsp.model.result.WzUpdateEntityResult;

public class WzResponseManager {
	
	public static WzResult parseResponse(CloseableHttpResponse response,String apiMethod) throws WzException{
		int statusCode = response.getStatusLine().getStatusCode();

		HttpEntity httpEntity = response.getEntity();
		String resultMsg;
		try {
			resultMsg = EntityUtils.toString(httpEntity);
		} catch (ParseException e1) {
			// TODO Auto-generated catch block
			e1.printStackTrace();
			throw new WzException(WzException.ParseException,e1.getMessage());
		} catch (IOException e1) {
			// TODO Auto-generated catch block
			e1.printStackTrace();
			throw new WzException(WzException.IOException,e1.getMessage());
		}
    	if(statusCode == 200){//网络请求成功返回结果
    		JSONObject resultJson = null;
    		try{
    			resultJson = new JSONObject(resultMsg);
    		}
    		catch (Exception e) {
    			throw new WzException("","");
    		}
    		
    		int code = resultJson.getInt("code");
    		if(apiMethod == WzConstants.CREATE_ENTITY){
    			WzCreateEntityResult wzResult = new WzCreateEntityResult();
    			wzResult.setCode(code);
    			if(resultJson.has("data"))
    				wzResult.setEntityName(resultJson.getString("data"));
    			if(resultJson.has("message"))
    				wzResult.setMessage(resultJson.getString("message"));
    			return wzResult;
    		}
    		else if(apiMethod == WzConstants.DELETE_ENTITY){
    			WzDeleteEntityResult wzResult = new WzDeleteEntityResult();
    			wzResult.setCode(code);
    			if(resultJson.has("message"))
    				wzResult.setMessage(resultJson.getString("message"));
    			return wzResult;
    		}
    		else if(apiMethod == WzConstants.UPDATE_ENTITY){
    			WzUpdateEntityResult wzResult = new WzUpdateEntityResult();
    			wzResult.setCode(code);
    			if(resultJson.has("message"))
    				wzResult.setMessage(resultJson.getString("message"));
    			return wzResult;
    		}
    		else if(apiMethod == WzConstants.QUERY_ENTITY){
    			WzQueryEntityResult wzResult = new WzQueryEntityResult();
    			wzResult.setCode(code);
    			if(resultJson.has("data")){
    				JSONObject entity = resultJson.getJSONObject("data");
    				WzEntity wzEntity = parseWzEntity(entity);
    				wzResult.setWzEntity(wzEntity);
    			}
    			if(resultJson.has("message"))
    				wzResult.setMessage(resultJson.getString("message"));
    			return wzResult;
    			}
    		else if(apiMethod == WzConstants.QUERY_ENTITIES){
    			WzQueryEntitiesResult wzResult = new WzQueryEntitiesResult();
				wzResult.setCode(code);	
				if(resultJson.has("data")){
					JSONArray entities = resultJson.getJSONArray("data");
    				List<WzEntity> wzEntities = parseWzEntities(entities);
    				wzResult.setWzEntities(wzEntities);
				}
				if(resultJson.has("message"))
    				wzResult.setMessage(resultJson.getString("message"));
    			return wzResult;
    			}
    		else if(apiMethod == WzConstants.QUERY_ENTITIES_BY_APP){
    			WzQueryEntitiesByAppResult wzResult = new WzQueryEntitiesByAppResult();
				wzResult.setCode(code);	
				if(resultJson.has("data")){
					JSONArray entities = resultJson.getJSONArray("data");
    				List<WzEntity> wzEntities = parseWzEntities(entities);
    				wzResult.setWzEntities(wzEntities);
				}
				if(resultJson.has("message"))
    				wzResult.setMessage(resultJson.getString("message"));
    			return wzResult;
    			}
    		
    		else if(apiMethod == WzConstants.ADD_POINT){
    			WzAddPointResult wzResult = new WzAddPointResult();
				wzResult.setCode(code);	
				if(resultJson.has("message"))
    				wzResult.setMessage(resultJson.getString("message"));
    			return wzResult;
    		}
    		else if(apiMethod == WzConstants.ADD_POINTS){
    			WzAddPointsResult wzResult = new WzAddPointsResult();
				wzResult.setCode(code);	
				if(resultJson.has("message"))
    				wzResult.setMessage(resultJson.getString("message"));
    			return wzResult;
    		}
    		else if(apiMethod == WzConstants.LOCATE){
    			WzLocateResult wzResult = new WzLocateResult();
				wzResult.setCode(code);	
				if(resultJson.has("data")){
					JSONArray points = resultJson.getJSONArray("data");
    				List<WzEntityPoint> wzEntityPoints = parseWzEntityPoints(points);
    				wzResult.setWzEntityPoints(wzEntityPoints);
				}
				if(resultJson.has("message"))
    				wzResult.setMessage(resultJson.getString("message"));
    			return wzResult;
    		}
    		else if(apiMethod == WzConstants.QUERY_POINTS){
    			WzQueryPointsResult wzResult = new WzQueryPointsResult();
				wzResult.setCode(code);	
				if(resultJson.has("data")){
					JSONArray points = resultJson.getJSONArray("data");
    				List<WzEntityPoint> wzEntityPoints = parseWzEntityPoints(points);
    				wzResult.setWzEntityPoints(wzEntityPoints);
				}
				if(resultJson.has("message"))
    				wzResult.setMessage(resultJson.getString("message"));
    			return wzResult;
    		}
     	}
    	else{
    		throw new WzException(WzException.WzServerException,resultMsg);
    	}
		return null;
	}
	
	private static WzEntity parseWzEntity(JSONObject entity){
		WzEntity wzEntity = new WzEntity();
		if(entity.has("bizType"))
			wzEntity.setBizType(entity.getString("bizType"));
		if(entity.has("entityName"))
			wzEntity.setEntityName(entity.getString("entityName"));
		if(entity.has("entityType"))
			wzEntity.setEntityType(entity.getString("entityType"));
		if(entity.has("attributes")){
			JSONObject attributes = entity.getJSONObject("attributes");
			if(attributes != null){
		        Iterator<String> iterator = attributes.keys();
		        while(iterator.hasNext()){
		            String key = (String) iterator.next();
		            Object value = attributes.get(key);
		            wzEntity.addAttribute(key, value);
		            }
			}
		}
		return wzEntity;
	}
	
	private static List<WzEntity> parseWzEntities(JSONArray entities){
		List<WzEntity> wzEntities = new ArrayList<WzEntity>(); 
		for(int i = 0; i < entities.length(); i++){  
			WzEntity wzEntity = parseWzEntity(entities.getJSONObject(i));
			wzEntities.add(wzEntity);
		}
		return wzEntities;
	}
	
	private static WzEntityPoint parseWzEntityPoint(JSONObject point){
		WzEntityPoint wzEntityPoint = new WzEntityPoint();
		if(point.has("entityName"))
			wzEntityPoint.setEntityName(point.getString("entityName"));
		if(point.has("lon"))
			wzEntityPoint.setLon(point.getDouble("lon"));
		if(point.has("lat"))
			wzEntityPoint.setLat(point.getDouble("lat"));
		if(point.has("coordinate"))
			wzEntityPoint.setCoordinateType(point.getString("coordinate"));
		if(point.has("gnssTime"))
			wzEntityPoint.setGnssTime(point.getLong("gnssTime"));
		if(point.has("alititude"))
			wzEntityPoint.setAltitude(point.getDouble("alititude"));
		if(point.has("speed"))
			wzEntityPoint.setSpeed(point.getDouble("speed"));
		if(point.has("direction"))
			wzEntityPoint.setDirection(point.getDouble("direction"));
		if(point.has("attributes")){
			JSONObject attributes = point.getJSONObject("attributes");
			if(attributes != null){
		        Iterator<String> iterator = attributes.keys();
		        while(iterator.hasNext()){
		            String key = (String) iterator.next();
		            Object value = attributes.get(key);
		            wzEntityPoint.addAttribute(key, value);
		            }
			}
		}
		return wzEntityPoint;
	}
	
	private static List<WzEntityPoint> parseWzEntityPoints(JSONArray points){
		List<WzEntityPoint> wzEntityPoints = new ArrayList<WzEntityPoint>(); 
		for(int i = 0; i < points.length(); i++){  
			WzEntityPoint wzEntityPoint = parseWzEntityPoint(points.getJSONObject(i));
			wzEntityPoints.add(wzEntityPoint);
		}
		return wzEntityPoints;
	}
	
}