package samples;

import java.util.ArrayList;

import java.util.List;

import com.qx.gpsp.WzQxyzClient;
import com.qx.gpsp.common.WzBizType;
import com.qx.gpsp.common.WzCoordinateType;
import com.qx.gpsp.common.WzException;
import com.qx.gpsp.manager.WzEntityManager;
import com.qx.gpsp.manager.WzEntityPointManager;
import com.qx.gpsp.model.WzEntity;
import com.qx.gpsp.model.WzEntityPoint;
import com.qx.gpsp.model.result.WzAddPointResult;
import com.qx.gpsp.model.result.WzAddPointsResult;
import com.qx.gpsp.model.result.WzCreateEntityResult;
import com.qx.gpsp.model.result.WzDeleteEntityResult;
import com.qx.gpsp.model.result.WzLocateResult;
import com.qx.gpsp.model.result.WzQueryPointsResult;

/**
 * 此示例为轨迹点管理（WzEntityPointManager）的全部函数调用示例
 */

public class Demo_WzEntityPointManager {
	public static void main(String[] args){
		System.out.println("-----------位置管理-begin-----------");
		//初始化云踪实例,请替换成您自己的appKey和appSecret
		WzQxyzClient client = null;
		try {
			client = new WzQxyzClient("appKey","appSecret");
		} catch (WzException e1) {
			// TODO Auto-generated catch block
			e1.printStackTrace();
		}
		WzEntityManager wzEntityManager = client.getWzEntityManager();
		WzEntityPointManager wzEntityPointManager = client.getWzEntityPointManager();

		String entityName = "实体-点测试";
		//创建实体A
		WzEntity wzEntity = new WzEntity();
		wzEntity.setEntityName(entityName);
		wzEntity.setEntityType("电动车");
		wzEntity.addAttribute("color", "黑色");
		wzEntity.addAttribute("driver", "千小寻");
		try {
			WzCreateEntityResult wzCreateEntityResult = wzEntityManager.createEntity(wzEntity);
			if(wzCreateEntityResult.getCode() == 0){
				System.out.println("创建实体成功："+wzCreateEntityResult.getEntityName());
			} else{
				System.out.println("创建实体失败：（"+wzCreateEntityResult.getCode()+"）"+wzCreateEntityResult.getMessage());
			}
		} catch (WzException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}

		//上传单个轨迹点
		WzEntityPoint wzEntityPoint = new WzEntityPoint();
		wzEntityPoint.setEntityName(entityName);
		wzEntityPoint.setLon(121.46);
		wzEntityPoint.setLat(31.45);
		wzEntityPoint.setCoordinateType(WzCoordinateType.WGS84);
		wzEntityPoint.setGnssTime(System.currentTimeMillis());
		wzEntityPoint.addAttribute("acc", "on");
		wzEntityPoint.addAttribute("temperature", 4.1);
		try {
			WzAddPointResult wzAddPointResult = wzEntityPointManager.addPoint(wzEntityPoint);
			if(wzAddPointResult.getCode() == 0){
				System.out.println("上传单个轨迹点成功："+entityName);
			} else{
				System.out.println("上传单个轨迹点失败：（"+wzAddPointResult.getCode()+"）"+wzAddPointResult.getMessage());
			}
		} catch (WzException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}

		//查询实时位置点
		List<String> entityNames = new ArrayList<String>();
		entityNames.add(entityName);
		try {
			WzLocateResult wzLocateResult = wzEntityPointManager.locate(WzBizType.GENERAL, entityNames, 180);
			if(wzLocateResult.getCode() == 0){
				System.out.println("查询实时位置点成功："+wzLocateResult.getWzEntityPoints().toString());
			} else{
				System.out.println("查询实时位置点失败：（"+wzLocateResult.getCode()+"）"+wzLocateResult.getMessage());
			}
		} catch (WzException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
			
		//批量上传轨迹点
		WzEntityPoint wzEntityPointA = new WzEntityPoint();
		wzEntityPointA.setEntityName(entityName);
		wzEntityPointA.setLon(121.04);
		wzEntityPointA.setLat(23.45);
		wzEntityPointA.setCoordinateType(WzCoordinateType.WGS84);
		wzEntityPointA.setGnssTime(System.currentTimeMillis()+10);
		wzEntityPointA.addAttribute("acc", "on");
		wzEntityPointA.addAttribute("temperature", 4.1);
		
		WzEntityPoint wzEntityPointB = new WzEntityPoint();
		wzEntityPointB.setEntityName(entityName);
		wzEntityPointB.setLon(120.03);
		wzEntityPointB.setLat(30.43);
		wzEntityPointB.setCoordinateType(WzCoordinateType.WGS84);
		wzEntityPointB.setGnssTime(System.currentTimeMillis()+20);
		wzEntityPointB.addAttribute("acc", "off");
		wzEntityPointB.addAttribute("temperature", 3.0);
		
		List<WzEntityPoint> wzEntityPoints = new ArrayList<WzEntityPoint>();
		wzEntityPoints.add(wzEntityPointA);
		wzEntityPoints.add(wzEntityPointB);

		try {
			WzAddPointsResult wzAddPointsResult = wzEntityPointManager.addPoints(wzEntityPoints);
			if(wzAddPointsResult.getCode() == 0){
				System.out.println("批量上传轨迹点成功："+entityName);
			} else{
				System.out.println("批量上传轨迹点失败：（"+wzAddPointsResult.getCode()+"）"+wzAddPointsResult.getMessage());
			}
		} catch (WzException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		
		//查询历史轨迹
		try {
			WzQueryPointsResult wzQueryPointsResult = wzEntityPointManager.queryPoints(WzBizType.GENERAL, entityName,
					System.currentTimeMillis()-24*60*60*1000, System.currentTimeMillis(), 1, 20);
			if(wzQueryPointsResult.getCode() == 0){
				System.out.println("查询历史轨迹成功："+"(轨迹点总数："+wzQueryPointsResult.getWzEntityPoints().size()+")"+wzQueryPointsResult.getWzEntityPoints().toString());
			} else{
				System.out.println("查询历史轨迹失败：（"+wzQueryPointsResult.getCode()+"）"+wzQueryPointsResult.getMessage());
			}	
		} catch (WzException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		
		//删除实体
		try {
			WzDeleteEntityResult wzDeleteEntityResult = wzEntityManager.deleteEntity(WzBizType.GENERAL,entityName);
			if(wzDeleteEntityResult.getCode() == 0){
				System.out.println("删除实体成功:"+"实体A");
			} else{
				System.out.println("删除实体失败：（"+wzDeleteEntityResult.getCode()+"）"+wzDeleteEntityResult.getMessage());
			}
		} catch (WzException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		
		System.out.println("-----------位置管理-end-----------");
	}
}