package samples.work;

import java.util.Timer;
import java.util.TimerTask;

import com.qx.gpsp.WzQxyzClient;
import com.qx.gpsp.common.WzBizType;
import com.qx.gpsp.common.WzCoordinateType;
import com.qx.gpsp.common.WzException;
import com.qx.gpsp.manager.WzEntityManager;
import com.qx.gpsp.manager.WzEntityPointManager;
import com.qx.gpsp.model.WzEntity;
import com.qx.gpsp.model.WzEntityPoint;
import com.qx.gpsp.model.result.WzAddPointResult;
import com.qx.gpsp.model.result.WzCreateEntityResult;

/**
 * Description
 * 此示例主要展示如何将现有平台上的位置数据与千寻云踪进行同步，采用弱同步方式的方式：
 * 所谓弱同步方案，是指两个平台的实体不保持严格同步，仅保持活跃的实体同步，千寻云踪中不存储不活跃的实体，千寻云踪不同步删除操作。
 *（1）开发者调动千寻云踪时，无需关注实体是否存在，直接调用上传单个轨迹点/批量上传轨迹点接口，如果成功，则处理完毕。相关接口：gpsp.point.addPoint、gpsp.point.addPoints。
 *（2）如果返回没有该实体的错误，则创建该实体后，创建实体成功后，再次上传轨迹点。相关接口：gpsp.entity.createEntity，gpsp.point.addPoint、gpsp.point.addPoints。
 * 
 * Link
 * 云踪漫谈-数据同步方案：https://bbs.qxwz.com/read.php?tid=376
 * 
 * Tips
 * 假设原有系统遵循终端-平台-数据库的设计方案，使用云踪的能力相当于平台-数据库环节新开一个分支（平台-千寻云踪），将数据写入云踪，然后平台按需从云踪获取能力。
 * 
 * 					终端            				 终端
 *  			 	 |               			  |
 * （原有业务系统系统） 平台             			     平台--云踪  （业务系统与云踪集成）
 *  			     |               			  |
 * 				   数据库           				 数据库
 *
 * 你需要自己来获取并构造实体，必须传递的字段有：实体名称（如车辆名称，手表IMEI号，作为唯一标识符）、实体类型（如大货车、儿童手表），其他字段按需自定义即可。
 * 您需要自己来获取并构造轨迹点，必须传递的字段有：实体名称（如车辆名称，手表IMEI号，作为唯一标识符）、经度、纬度、gnss时间，其他字段按需自定义即可。
 */
public class Demo_Platform2Platform {
	public static void main(String[] args){
		//初始化云踪实例,请替换成您自己的appKey和appSecret
		WzQxyzClient client = null;
		try {
			client = new WzQxyzClient("appKey","appSecret");
		} catch (WzException e1) {
			// TODO Auto-generated catch block
			e1.printStackTrace();
		}
		WzEntityManager wzEntityManager = client.getWzEntityManager();
		WzEntityPointManager wzEntityPointManager = client.getWzEntityPointManager();
		    
		//上传单个轨迹点
		String entityName = "testA";
		WzEntityPoint wzEntityPoint = new WzEntityPoint();
		wzEntityPoint.setEntityName(entityName);//设置实体的名称
		wzEntityPoint.setLon(121.46);//设置经度
		wzEntityPoint.setLat(31.45);//设置纬度
		wzEntityPoint.setCoordinateType(WzCoordinateType.WGS84);//设置坐标类型，支持WGS84和GCJ02两种。
		wzEntityPoint.setGnssTime(System.currentTimeMillis());//设置坐标时间
		wzEntityPoint.addAttribute("acc", "on");//按需设置自定义属性
		wzEntityPoint.addAttribute("temperature", 4.1);//按需设置自定义属性
		try {
			WzAddPointResult wzAddPointResult = wzEntityPointManager.addPoint(wzEntityPoint);
			if(wzAddPointResult.getCode() == 0){
				System.out.println("上传单个轨迹点成功："+entityName);
			} else{
				System.out.println("上传单个轨迹点失败：（"+wzAddPointResult.getCode()+"）"+wzAddPointResult.getMessage());
				if(wzAddPointResult.getCode() == 20202){//实体不存在后，创建实体后重新上传点。
					//创建实体A
					WzEntity wzEntity = new WzEntity();
					wzEntity.setEntityName(entityName);//设置实体名称
					wzEntity.setEntityType("电动车");//设置实体类型
					wzEntity.addAttribute("color", "黑色");//按需设置自定义属性
					wzEntity.addAttribute("driver", "千小寻");//按需设置自定义属性
					try {
						WzCreateEntityResult wzCreateEntityResult = wzEntityManager.createEntity(wzEntity);
						if(wzCreateEntityResult.getCode() == 0){
							System.out.println("创建实体成功："+wzCreateEntityResult.getEntityName());
							try {
								WzAddPointResult wzAddPointResult2 = wzEntityPointManager.addPoint(wzEntityPoint);
								if(wzAddPointResult2.getCode() == 0){
									System.out.println("上传单个轨迹点成功："+entityName);
								}else{
									System.out.println("上传单个轨迹点失败：（"+wzAddPointResult2.getCode()+"）"+wzAddPointResult2.getMessage());
								}
							} catch (WzException e) {
								// TODO Auto-generated catch block
								e.printStackTrace();
							}
						} else{
							System.out.println("创建实体失败：（"+wzCreateEntityResult.getCode()+"）"+wzCreateEntityResult.getMessage());
						}
					} catch (WzException e) {
						// TODO Auto-generated catch block
						e.printStackTrace();
					}
				}	
			}
		} catch (WzException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		
		//定时上传轨迹点示例
		/*Timer timer = new Timer();
        TimerTask tt=new TimerTask() { 
            @Override
            public void run() {
            	//上传单个轨迹点
        		String entityName = "test"+(int)(20*Math.random());
        		WzEntityPoint wzEntityPoint = new WzEntityPoint();
        		wzEntityPoint.setEntityName(entityName);//设置实体的名称
        		wzEntityPoint.setLon(121.46-Math.random());//设置经度
        		wzEntityPoint.setLat(31.45-Math.random());//设置纬度
        		wzEntityPoint.setCoordinateType(WzCoordinateType.WGS84);//设置坐标类型，支持WGS84和GCJ02两种。
        		wzEntityPoint.setGnssTime(System.currentTimeMillis());//设置坐标时间
        		wzEntityPoint.addAttribute("acc", "on");//设置自定义属性
        		wzEntityPoint.addAttribute("temperature", 4.1);//设置自定义属性
        		try {
        			WzAddPointResult wzAddPointResult = wzEntityPointManager.addPoint(wzEntityPoint);
        			if(wzAddPointResult.getCode() == 0){
        				System.out.println("上传单个轨迹点成功："+entityName);
        			} else{
        				System.out.println("上传单个轨迹点失败：（"+wzAddPointResult.getCode()+"）"+wzAddPointResult.getMessage());
        				if(wzAddPointResult.getCode() == 20202){//实体不存在后，重新创建实体后重新上传点。
        					//创建实体A
        					WzEntity wzEntity = new WzEntity();
        					wzEntity.setBizType(WzBizType.GENERAL);
        					wzEntity.setEntityName(entityName);//设置实体名称
        					wzEntity.setEntityType("电动车");//设置实体类型
        					wzEntity.addAttribute("color", "黑色");//设置自定义属性
        					wzEntity.addAttribute("driver", "千小寻");//设置自定义属性
        					try {
        						WzCreateEntityResult wzCreateEntityResult = wzEntityManager.createEntity(wzEntity);
        						if(wzCreateEntityResult.getCode() == 0){
        							System.out.println("创建实体成功："+wzCreateEntityResult.getEntityName());
        								try {
        								WzAddPointResult wzAddPointResult2 = wzEntityPointManager.addPoint(wzEntityPoint);
        								if(wzAddPointResult2.getCode() == 0){
        									System.out.println("上传单个轨迹点成功："+entityName);
        								}else{
        									System.out.println("上传单个轨迹点失败：（"+wzAddPointResult2.getCode()+"）"+wzAddPointResult2.getMessage());
        								}
        							} catch (WzException e) {
        								// TODO Auto-generated catch block
        								e.printStackTrace();
        							}
        						} else{
        							System.out.println("创建实体失败：（"+wzCreateEntityResult.getCode()+"）"+wzCreateEntityResult.getMessage());
        						}
        					} catch (WzException e) {
        						// TODO Auto-generated catch block
        						e.printStackTrace();
        					}
        				}	
        			}
        		} catch (WzException e) {
        			// TODO Auto-generated catch block
        			e.printStackTrace();
        		}
            }
        };
        timer.schedule(tt, 1000,2000);*/
	}
}